SystemOrganization addCategory: #'Zinc-WebSocket-Core'!

ZnResponse subclass: #ZnWebSocketResponse
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketResponse commentStamp: '<historical>' prior: 0!
I am ZnWebSocketResponse used to continue the WebSocket conversation after the connection upgrade.
I am a ZnResponse.

Part of Zinc HTTP Components.!

!ZnWebSocketResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 13:33'!
handler
	^ handler! !

!ZnWebSocketResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/22/2012 13:33'!
handler: block
	"Set the block that will deal with the server side WebSocket"
	
	handler := block! !

!ZnWebSocketResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 14:31'!
useConnection: connection
	"Hook method overwritten to give the receiver the chance to 
	keep using connection in the current thread/process after the server wrote the response.
	We start a conversation by instanciating a WebSocket and passing it to our handler."
	
	| webSocket message |
	webSocket := ZnWebSocket onStream: connection.
	self handler value: webSocket! !

!ZnWebSocketResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/20/2012 16:58'!
wantsConnectionClose
	"After we are done, the server can close the connection we were using"
	 
	^ true! !

ConnectionClosed subclass: #ZnWebSocketClosed
	instanceVariableNames: 'code reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketClosed commentStamp: '<historical>' prior: 0!
I am ZnWebSocketClosed, signalled when WebSocket close frame is received.
I am a ConnectionClosed and a NetworkError.

Part of Zinc HTTP Components.
!

!ZnWebSocketClosed methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:56'!
code
	^ code! !

!ZnWebSocketClosed methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:56'!
code: anObject
	code := anObject! !

!ZnWebSocketClosed methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:56'!
reason
	^ reason! !

!ZnWebSocketClosed methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:56'!
reason: anObject
	reason := anObject! !

Error subclass: #ZnWebSocketFailed
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketFailed commentStamp: '<historical>' prior: 0!
I am ZnWebSocketFailed, an exception thrown when a client WebSocket cannot be created successfully.
I am an Error.

Part of Zinc HTTP Components. !

!ZnWebSocketFailed class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/19/2012 15:47'!
response: response
	^ self new
		response: response;
		yourself! !

!ZnWebSocketFailed methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/19/2012 15:46'!
response
	^ response! !

!ZnWebSocketFailed methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/19/2012 15:46'!
response: anObject
	response := anObject! !

Object subclass: #ZnWebSocket
	instanceVariableNames: 'stream role'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocket commentStamp: '<historical>' prior: 0!
I am ZnWebSocket, a bi-directional, full-duplex communication channel over a single TCP connection.

WebSockets send reliable messages to each other, a String or ByteArrat.
Each message is one-way (does not have a direct synchroneous reply). 
Each party, both client and/or server can take the initiative.

This implementation follows RFC 6455. See also:

	http://en.wikipedia.org/wiki/Websockets
	http://tools.ietf.org/html/rfc6455

Here is a client side example taking to a public echo service: 

| webSocket |
webSocket := ZnWebSocket to: 'ws://echo.websocket.org'.
[ webSocket 
	sendMessage: 'Pharo Smalltalk using Zinc WebSockets !!';
	readMessage ] ensure: [ webSocket close ].

Here is the same example using secure web sockets:

| webSocket |
webSocket := ZnWebSocket to: 'wss://echo.websocket.org'.
[ webSocket 
	sendMessage: 'Pharo Smalltalk using Zinc WebSockets & Zodiac !!';
	readMessage ] ensure: [ webSocket close ].

Here is how to start a local server running in the image implementing the same echo service:

ZnWebSocket startServerOn: 8080 do: ZnWebSocketEchoHandler new.

A full inline implementation of the echo handler would look like:

ZnWebSocket 
	startServerOn: 8080 
	do: [ :webSocket | 
		[ 
			webSocket runWith: [ :message |
				self crLog: 'Received message: ', message printString, ' echoing...'.
				webSocket sendMessage: message ] ] 
			on: ConnectionClosed 
			do: [ self crLog: 'Ignoring connection close, done' ] ].

Part of Zinc HTTP Components.!

!ZnWebSocket class methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/2/2012 20:35'!
isValidWebSocketResponse: client
	"Return true when client contains a valid WebSocket response for its WebSocket request"

	| acceptKey response |
	acceptKey := ZnWebSocketUtils handshake: (client request headers at: 'Sec-WebSocket-Key').
	response := client response.
	^ response code = 101
		and: [
			(response headers at: 'Upgrade') asLowercase = 'websocket' 
				and: [
					(ZnWebSocketUtils containsConnectionUpgrade: response headers)
						and: [ 
							(response headers at: 'Sec-WebSocket-Accept') = acceptKey ] ] ]! !

!ZnWebSocket class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/2/2012 19:04'!
onStream: socketStream
	"Return a new ZnWebSocket instance on socketStream.
	It is assumed that the initial handshake was successful."
	
	^ self new
		onStream: socketStream;
		yourself! !

!ZnWebSocket class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/2/2012 20:33'!
startServerOn: port do: handler
	"Start and return a new ZnServer listening on port and / for client WebSocket connections.
	After a successful initial handshake, sent #value: to handler with a server side ZnWebSocket instance."
	
	^ self startServerOn: port prefix: nil do: handler  ! !

!ZnWebSocket class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/2/2012 20:33'!
startServerOn: port prefix: prefix do: handler
	"Start and return a new ZnServer listening on port and /prefix for client WebSocket connections.
	After a successful initial handshake, sent #value: to handler with a server side ZnWebSocket instance."
	
	^ (ZnServer on: port)
		delegate: (ZnWebSocketDelegate map: prefix to: handler);
		start;
		yourself! !

!ZnWebSocket class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/2/2012 19:03'!
to: urlObject
	"Attempt to create a new client WebSocket to urlObject.
	Do the initial upgrade handshake and return a functioning ZnWebSocket object.
	Signals a ZnWebSocketFailed error when unsuccessful."
	
	| client |
	client := self webSocketClientTo: urlObject asZnUrl.
	client execute.
	(self isValidWebSocketResponse: client)
		ifTrue: [
			^ (self onStream: client connection)
				role: #client;
				yourself ]
		ifFalse: [
			client close.
			(ZnWebSocketFailed response: client response) signal ]! !

!ZnWebSocket class methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/2/2012 20:34'!
webSocketClientTo: url
	"Create and return a new ZnClient instance ready for the initial client side WebSocket setup request"
	
	| client |
	self assert: (#(ws wss) includes: url scheme).
	(client := ZnClient new)
		url: url;
		method: #GET;
		headerAt: 'Upgrade' put: 'websocket';
		headerAt: 'Connection' put: 'Upgrade';
		headerAt: 'Sec-WebSocket-Version' put: '13';
		headerAt: 'Sec-WebSocket-Key' put: ZnWebSocketUtils newClientKey.
	^ client   
	! !

!ZnWebSocket methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/21/2012 20:43'!
close
	"Send a close frame and close the connection"

	self sendFrame: ZnWebSocketFrame close.
	stream close.! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/23/2012 12:02'!
handleControlFrame: frame
	"Handle a control frame. ConnectionClosed is signalled when the other end closes."
	
	frame isClose 
		ifTrue: [ 
			self crLog: 'Handling close frame'.
			self close. 
			^ self signalClosed: frame ].
	frame isPing 
		ifTrue: [ 
			self crLog: 'Handling ping frame'.
			self pong: frame body ].
	frame isPong 
		ifTrue: [
			self crLog: 'Handling pong frame' ]! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/23/2012 11:25'!
handleControlFrameNoWait
	"Peek for a control frame if any and handle it. Do not wait.
	Return true when a control frame was handled, false otherwise."
	
	(stream isDataAvailable and: [ (stream peek bitAnd: 127) > 7 ])
		ifTrue: [ | frame |
			frame := ZnWebSocketFrame readFrom: stream.
			self handleControlFrame: frame.
			^ true ].
	^ false! !

!ZnWebSocket methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/2/2012 20:35'!
initialize
	super initialize.
	role := #server! !

!ZnWebSocket methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/12/2012 15:04'!
isClient
	"Return true if the receiver is the client in a WebSocket conversation"
	
	^ role = #client! !

!ZnWebSocket methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/12/2012 15:05'!
isConnected
	"Return true when I am connected, 
	i.e. not closed and still connected at the socket level"

	^ stream notNil and: [ stream isConnected ]! !

!ZnWebSocket methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/12/2012 15:04'!
isServer
	"Return true if the receiver is the server in a WebSocket conversation"

	^ role = #server! !

!ZnWebSocket methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/2/2012 20:36'!
onStream: socketStream
	"Initialize the receiver on socketStream"
	
	stream := socketStream ! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/23/2012 11:31'!
ping
	"Send a ping frame."

	| frame |
	frame := ZnWebSocketFrame ping: ZnWebSocketUtils newPingPayload.
	self sendFrame: frame! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/23/2012 11:31'!
pong: payload
	"Send a pong frame containing payload (in reply to a ping with that payload)"
	
	| frame |
	frame := ZnWebSocketFrame pong: payload.
	self sendFrame: frame ! !

!ZnWebSocket methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/21/2012 16:27'!
printOn: aStream
	super printOn: aStream.
	aStream 
		nextPut: $(; 
		print: role;
		nextPut: $)! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/7/2012 13:42'!
readFrame
	"Read and return the next non-control frame. Wait.
	Control frames are handled. ConnectionClosed is signalled when the other end closes."
	
	| frame |
	frame := ZnWebSocketFrame readFrom: stream.
	frame isControl 
		ifTrue: [
			self handleControlFrame: frame.
			^ self readFrame ].
	^ frame! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/12/2012 15:13'!
readMessage
	"Read and return a complete message String or ByteArray, joining frames. 
	ConnectionClosed is signalled when the other end closes."
	
	| frame bytes isText |
	frame := self readFrame.
	frame isFinal ifTrue: [ ^ frame contents ].
	isText := frame isText.
	bytes := ByteArray new: frame size * 2 streamContents: [ :output |
		frame writeRawBodyOn: output.
		[ 
			frame := self readFrame.
			self assert: frame isContinuation. 
			frame writeRawBodyOn: output.
			frame isFinal ] whileFalse ].
	^ isText 
		ifTrue: [ ZnUTF8Encoder new decodeBytes: bytes ]
		ifFalse: [ bytes ]! !

!ZnWebSocket methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 16:11'!
role
	"Return the role of the receiver, either #client or #server"
	
	^ role! !

!ZnWebSocket methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/21/2012 16:12'!
role: symbol
	"Set the role of the receiver, either #client or #server"
	
	self assert: (#(client server) includes: symbol).
	role := symbol! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/7/2012 13:41'!
runWith: block
	"Start a run loop handling the WebSocket protocol.
	When a message is received, block will be passed a String or ByteArray.
	ConnectionClosed will be signalled when the other end closes."
	
	[ | message | 
		[ message := self readMessage ] 
			on: ConnectionTimedOut 
			do: [ "Ignore & continue" nil ].
		message ifNotNil: [
			block value: message ] ] repeat! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/21/2012 22:11'!
sendByteFrames: sequenceableCollection
	"Send a binary message consisting of sequenceableCollection byte array frames"

	| frame |
	self assert: sequenceableCollection size > 1.
	(frame := ZnWebSocketFrame bytes: sequenceableCollection first)
		final: false.
	self sendFrame: frame.
	2 to: sequenceableCollection size - 1 do: [ :each |
		(frame := ZnWebSocketFrame bytes: (sequenceableCollection at: each))
			final: false; opcode: 0.
		self sendFrame: frame ].
	(frame := ZnWebSocketFrame bytes: sequenceableCollection last)
		final: true; opcode: 0.
	self sendFrame: frame
	! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/21/2012 21:18'!
sendBytes: bytes
	"Send bytes in one binary frame"

	| frame |
	frame := ZnWebSocketFrame bytes: bytes.
	self sendFrame: frame! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/2/2012 20:38'!
sendFrame: frame
	"Send a single frame, masked when I am in a client role, flush"
	
	frame masked: self isClient.
	frame writeOn: stream.
	stream flush.! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/21/2012 21:22'!
sendMessage: stringOrByteArray
	"Send stringOrByteArray in one textual or binary frame"

	stringOrByteArray isString
		ifTrue: [ self sendText: stringOrByteArray ] 
		ifFalse: [ self sendBytes: stringOrByteArray ]! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/21/2012 20:45'!
sendText: string
	"Send string in one textual frame"

	| frame |
	frame := ZnWebSocketFrame text: string.
	self sendFrame: frame! !

!ZnWebSocket methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/21/2012 22:11'!
sendTextFrames: sequenceableCollection
	"Send a textual message consisting of sequenceableCollection string frames"

	| frame |
	self assert: sequenceableCollection size > 1.
	(frame := ZnWebSocketFrame text: sequenceableCollection first)
		final: false.
	self sendFrame: frame.
	2 to: sequenceableCollection size - 1 do: [ :each |
		(frame := ZnWebSocketFrame text: (sequenceableCollection at: each))
			final: false; opcode: 0.
		self sendFrame: frame ].
	(frame := ZnWebSocketFrame text: sequenceableCollection last)
		final: true; opcode: 0.
	self sendFrame: frame
	! !

!ZnWebSocket methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/23/2012 12:03'!
signalClosed: frame
	"Turn a close frame that we received into a ZnWebSocketClosed exception and signal it."
	
	| exception |
	exception := ZnWebSocketClosed new.
	frame isEmpty 
		ifFalse: [
			exception 
				code: frame closeCode;
				reason: frame closeReason ].
	exception signal.! !

!ZnWebSocket methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/20/2012 15:50'!
stream
	^ stream! !

Object subclass: #ZnWebSocketChatroomHandler
	instanceVariableNames: 'webSockets lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketChatroomHandler commentStamp: '<historical>' prior: 0!
I am ZnWebSocketChatroomHandler, I send a copy of each incoming message over a WebSocket to all WebSockets currently open, thus implementing a simple chat room.

See the class comment of ZnWebSocketDelegate for an example using ZnWebSocketChatroomHandler.

Part of Zinc HTTP Components.!

!ZnWebSocketChatroomHandler class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 9/7/2012 21:48'!
clientHtml
	^ '<!!DOCTYPE html>
<meta charset="utf-8" />
<title>WebSocket Chatroom</title>

<script language="javascript" type="text/javascript">

  var wsUri = "ws://localhost:1701/ws-chatroom";
  var websocket;
  var output, handleInput, messageInput, sendButton, closeButton;

  function init() {
    output = document.getElementById("output");
    handleInput = document.getElementById("handleInput");
    messageInput = document.getElementById("messageInput");
    sendButton = document.getElementById("sendButton");
    closeButton = document.getElementById("closeButton");
    handleInput.value = "user-" + (new Date()).getMilliseconds();
    messageInput.onkeyup = function (evt) { if (evt.keyCode == 13) sendChatMessage(evt) }
    sendButton.onclick = function (evt) { sendChatMessage(evt) };
    closeButton.onclick = function (evt) { closeChat(evt) };
    testWebSocket();
  }

  function testWebSocket() {
    websocket = new WebSocket(wsUri);
    websocket.onopen = function(evt) { onOpen(evt) };
    websocket.onclose = function(evt) { onClose(evt) };
    websocket.onmessage = function(evt) { onMessage(evt) };
    websocket.onerror = function(evt) { onError(evt) };
  }

  function onOpen(evt) {
    writeToScreen("CONNECTED");
    doSend("Hi there !!");
  }

  function onClose(evt) {
    writeToScreen("DISCONNECTED");
  }

  function onMessage(evt) {
    writeToScreen(evt.data);
   }

  function onError(evt) {
    writeToScreen("<span style=''color: red''>ERROR:</span> " + evt.data);
  }

  function doSend(message) {
     websocket.send(handleInput.value + ">>" + message);
  }

  function writeToScreen(message) {
    var pre = document.createElement("p");
    pre.style.wordWrap = "break-word";
    pre.innerHTML = message;
    output.appendChild(pre);
  }

  function sendChatMessage(evt) {
    doSend(messageInput.value);
    messageInput.value = "";
  }

  function closeChat(evt) {
    doSend("Bye!!");
    websocket.close();
  }

  window.addEventListener("load", init, false);

</script>

<h2>WebSocket Chatroom</h2>
<div>
Handle: <input id="handleInput" type="text"/>
Message: <input id="messageInput" type="text" size="64"/> 
<input id="sendButton" type="submit" value="Send"/>
<input id="closeButton" type="submit" value="Close"/>
</div>
<div id="output"></div>

</html> '
! !

!ZnWebSocketChatroomHandler methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/10/2012 12:01'!
distributeMessage: message 
	"Send message to all client WebSockets that we know."
	
	lock critical: [
		webSockets do: [ :each |
			[ each sendMessage: message ]
				on: NetworkError 
				do: [ :exception |
					"We can ignore this (instead of #unregister:-ing the offender), 
					since the listener process (see #value:) will do the right thing" 
					self crLog: exception printString, ' in ditributeMessage, ignoring' ] ] ]! !

!ZnWebSocketChatroomHandler methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/10/2012 11:33'!
initialize
	webSockets := OrderedCollection new.
	lock := Mutex new! !

!ZnWebSocketChatroomHandler methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/10/2012 11:35'!
register: clientWebSocket
	lock critical: [
		webSockets add: clientWebSocket ]! !

!ZnWebSocketChatroomHandler methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/10/2012 11:36'!
unregister: clientWebSocket
	lock critical: [
		webSockets remove: clientWebSocket ifAbsent: [ ] ]! !

!ZnWebSocketChatroomHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2012 11:37'!
value: webSocket
	"I implement a simple chat room service as a server:
	distributing incoming messages to all known clients"
	
	[
		self register: webSocket.
		webSocket runWith: [ :message |
			self crLog: 'Received message: ', message printString.
			self distributeMessage: message ] ] 
		on: ConnectionClosed 
		do: [
			self crLog: 'Connection close, cleaning up'.
			self unregister: webSocket ]! !

Object subclass: #ZnWebSocketDelegate
	instanceVariableNames: 'prefix handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketDelegate commentStamp: '<historical>' prior: 0!
I am ZnWebSocketDelegate, a plugin to allow a ZnServer to accept incoming WebSocket requests.

When accepted, the request is upgraded and a new WebSocket is created and handed to the handler block.

===
ZnServer startDefaultOn: 1701.
ZnServer default logToTranscript.
ZnServer default delegate
	map: 'ws-echo-client-remote' 
	to: [ :request | ZnResponse ok: (ZnEntity html: ZnWebSocketEchoHandler clientHtmlRemote) ];
	map: 'ws-echo-client' 
	to: [ :request | ZnResponse ok: (ZnEntity html: ZnWebSocketEchoHandler clientHtml) ];
	map: 'ws-echo'
	to: (ZnWebSocketDelegate map: 'ws-echo' to: ZnWebSocketEchoHandler new);
	map: 'ws-chatroom-client' 
	to: [ :request | ZnResponse ok: (ZnEntity html: ZnWebSocketChatroomHandler clientHtml) ];
	map: 'ws-chatroom'
	to: (ZnWebSocketDelegate map: 'ws-chatroom' to: ZnWebSocketChatroomHandler new);
	map: 'ws-status-client' 
	to: [ :request | ZnResponse ok: (ZnEntity html: ZnWebSocketStatusHandler clientHtml) ];
	map: 'ws-status'
	to: (ZnWebSocketDelegate map: 'ws-status' to: ZnWebSocketStatusHandler new).
ZnServer stopDefault.
===

Part of Zinc HTTP Components.!

!ZnWebSocketDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/11/2012 10:41'!
handler: handler
	"Create and return a new ZnWebSocketDelegate that maps / to handler"

	^ self new
		handler: handler;
		yourself! !

!ZnWebSocketDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/27/2012 20:59'!
map: prefix to: handler
	"Create and return a new ZnWebSocketDelegate that maps prefix string to handler"

	^ self new
		prefixFromString: prefix;
		handler: handler;
		yourself! !

!ZnWebSocketDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/22/2012 13:09'!
handleRequest: request
	"Server delegate entry point"
	
	(prefix isNil or: [ prefix = request uri pathSegments ])
		ifFalse: [ ^ ZnResponse notFound: request uri ].
	^ (self isValidWebSocketRequest: request)
		ifTrue: [ self webSocketResponseForRequest: request ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnWebSocketDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 13:06'!
handler
	^ handler! !

!ZnWebSocketDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/22/2012 13:33'!
handler: block
	"Set the block that will deal with the server side WebSocket"
	
	handler := block! !

!ZnWebSocketDelegate methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/7/2012 14:15'!
isValidWebSocketRequest: request
	"Return true when request can be considered a valid WebSocket setup request"
	
	^ request method = #GET
		and: [ 
			(request headers at: 'Upgrade' ifAbsent: [ ^ false ]) asLowercase = 'websocket' 
				and: [
					(ZnWebSocketUtils containsConnectionUpgrade: request headers) 
						and: [ 
							(request headers at: 'Sec-WebSocket-Version' ifAbsent: [ ^ false ]) = '13'
								and: [ 
									request headers includesKey: 'Sec-WebSocket-Key' ] ] ] ]! !

!ZnWebSocketDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 13:06'!
prefix
	^ prefix! !

!ZnWebSocketDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/22/2012 13:06'!
prefix: orderedCollection
	prefix := orderedCollection! !

!ZnWebSocketDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/22/2012 13:07'!
prefixFromString: string
	"Set the path prefix under which I should be handling request to string,
	interpreting each /-separated token as a path element"

	self prefix: (string findTokens: '/')! !

!ZnWebSocketDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/2/2012 20:40'!
responseHeadersForKey: key
	"Return the headers for a WebSocket setup response, given key"
	
	^ ZnHeaders defaultResponseHeaders
			at: 'Upgrade' put: 'websocket';
			at: 'Connection' put: 'Upgrade';
			at: 'Sec-WebSocket-Accept' put: key;
			yourself! !

!ZnWebSocketDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/22/2012 14:40'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

!ZnWebSocketDelegate methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 9/2/2012 20:41'!
webSocketResponseForRequest: request
	"Given a valid WebSocket setup request, return the matching server response"
	
	| acceptKey response |
	acceptKey := ZnWebSocketUtils handshake: (request headers at: 'Sec-WebSocket-Key').
	^ ZnWebSocketResponse new
		statusLine: (ZnStatusLine code: 101);
		headers: (self responseHeadersForKey: acceptKey);
		handler: self handler;
		yourself
		! !

Object subclass: #ZnWebSocketEchoHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketEchoHandler commentStamp: '<historical>' prior: 0!
I am ZnWebSocketEchoHandler, implementing an echo service conversation as a WebSocket server: reading messages and echoing them back until ConnectionClosed.

See the class comment of ZnWebSocketDelegate for an example using ZnWebSocketEchoHandler.

Part of Zinc HTTP Components.!

!ZnWebSocketEchoHandler class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 9/7/2012 15:43'!
clientHtml
	^ '<!!DOCTYPE html>
<meta charset="utf-8" />
<title>WebSocket Local Test (ws://localhost:1701/ws-echo)</title>

<script language="javascript" type="text/javascript">

  var wsUri = "ws://localhost:1701/ws-echo";
  var output;

  function init() {
    output = document.getElementById("output");
    testWebSocket();
  }

  function testWebSocket() {
    websocket = new WebSocket(wsUri);
    websocket.onopen = function(evt) { onOpen(evt) };
    websocket.onclose = function(evt) { onClose(evt) };
    websocket.onmessage = function(evt) { onMessage(evt) };
    websocket.onerror = function(evt) { onError(evt) };
  }

  function onOpen(evt) {
    writeToScreen("CONNECTED");
    doSend("WebSocket rocks");
  }

  function onClose(evt) {
    writeToScreen("DISCONNECTED");
  }

  function onMessage(evt) {
    writeToScreen("<span style=''color: blue;''>RESPONSE: " + evt.data+"</span>");
    websocket.close();
  }

  function onError(evt) {
    writeToScreen("<span style=''color: red''>ERROR:</span> " + evt.data);
  }

  function doSend(message) {
    writeToScreen("SENT: " + message); 
    websocket.send(message);
  }

  function writeToScreen(message) {
    var pre = document.createElement("p");
    pre.style.wordWrap = "break-word";
    pre.innerHTML = message;
    output.appendChild(pre);
  }

  window.addEventListener("load", init, false);

</script>

<h2>WebSocket Local Test (ws://localhost:1701/ws-echo)</h2>
<div id="output"></div>

</html> '! !

!ZnWebSocketEchoHandler class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 9/7/2012 15:42'!
clientHtmlRemote
	^ '<!!DOCTYPE html>
<meta charset="utf-8" />
<title>WebSocket Test (ws://echo.websocket.org)</title>

<script language="javascript" type="text/javascript">

  var wsUri = "ws://echo.websocket.org";
  var output;

  function init() {
    output = document.getElementById("output");
    testWebSocket();
  }

  function testWebSocket() {
    websocket = new WebSocket(wsUri);
    websocket.onopen = function(evt) { onOpen(evt) };
    websocket.onclose = function(evt) { onClose(evt) };
    websocket.onmessage = function(evt) { onMessage(evt) };
    websocket.onerror = function(evt) { onError(evt) };
  }

  function onOpen(evt) {
    writeToScreen("CONNECTED");
    doSend("WebSocket rocks");
  }

  function onClose(evt) {
    writeToScreen("DISCONNECTED");
  }

  function onMessage(evt) {
    writeToScreen("<span style=''color: blue;''>RESPONSE: " + evt.data+"</span>");
    websocket.close();
  }

  function onError(evt) {
    writeToScreen("<span style=''color: red''>ERROR:</span> " + evt.data);
  }

  function doSend(message) {
    writeToScreen("SENT: " + message); 
    websocket.send(message);
  }

  function writeToScreen(message) {
    var pre = document.createElement("p");
    pre.style.wordWrap = "break-word";
    pre.innerHTML = message;
    output.appendChild(pre);
  }

  window.addEventListener("load", init, false);

</script>

<h2>WebSocket Test (ws://echo.websocket.org)</h2>
<div id="output"></div>

</html> '! !

!ZnWebSocketEchoHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:41'!
value: webSocket
	"I implement an echo service conversation as a server:
	reading messages and echoing them back until ConnectionClosed"
	
	[ 
		webSocket runWith: [ :message |
			self crLog: 'Received message: ', message printString, ' echoing...'.
			webSocket sendMessage: message ] ] 
		on: ConnectionClosed 
		do: [ self crLog: 'Ignoring connection close, done' ]! !

Object subclass: #ZnWebSocketFrame
	instanceVariableNames: 'opcode final masked body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketFrame commentStamp: '<historical>' prior: 0!
I am ZnWebSocketFrame, the elementary, low-level unit of data transfer in the WebSocket protocol.

A frame has an opcode (a type), a flag indicating whether it is the final frame in a sequence, a flag indicating whether it is masked and a body (a ByteArray).

The wire encoding of a frame consists minimally of 2 bytes: the first holds the opcode and final bit, the second the size and masked bit. Sizes strictly smaller than 126 are encoded directly. If the first byte is 126, the size is encoded in 2 successive bytes. If the first byte is 127, the size is encoded in 8 successive bytes. Client to server messages are masked. A 4 byte mask is XOR-ed with all body bytes. See #readFrom: and #writeOn: for the details.

This implementation follows RFC 6455. See also:

	http://en.wikipedia.org/wiki/Websockets
	http://tools.ietf.org/html/rfc6455

Part of Zinc HTTP Components.!

!ZnWebSocketFrame class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/21/2012 15:22'!
bytes: byteArray
	^ self new
		opcode: 2;
		final: true;
		body: byteArray;
		yourself! !

!ZnWebSocketFrame class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/21/2012 15:26'!
close
	^ self new
		opcode: 8;
		final: true;
		yourself! !

!ZnWebSocketFrame class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/21/2012 20:22'!
ping: bytes
	^ self new
		opcode: 9;
		final: true;
		body: bytes;
		yourself! !

!ZnWebSocketFrame class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/21/2012 20:22'!
pong: bytes
	^ self new
		opcode: 10;
		final: true;
		body: bytes;
		yourself! !

!ZnWebSocketFrame class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/21/2012 15:40'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnWebSocketFrame class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/21/2012 20:30'!
text: string
	^ self new
		opcode: 1;
		final: true;
		text: string;
		yourself! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
body
	^ body! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
body: anObject
	body := anObject! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:52'!
closeCode
	self assert: self isClose.
	self assert: self isEmpty not.
	^ (body readStream next: 2) asInteger
	! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 11:53'!
closeReason
	self assert: self isClose.
	self assert: self isEmpty not.
	^ ZnUTF8Encoder new decodeBytes: (body allButFirst: 2)
	! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 10:28'!
contents
	self assert: self isControl not.
	^ self isText
		ifTrue: [ self text ]
		ifFalse: [ body ]! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
final
	^ final! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
final: anObject
	final := anObject! !

!ZnWebSocketFrame methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/21/2012 15:31'!
initialize
	super initialize.
	final := true! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 16:26'!
isBinary
	^ opcode = 2! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 17:10'!
isClose
	^ opcode = 8! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 17:10'!
isContinuation
	^ opcode = 0! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:46'!
isControl
	^ opcode > 7! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 21:00'!
isEmpty
	^ self size = 0! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 20:34'!
isFinal
	^ final! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 22:29'!
isMasked
	^ masked! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 17:09'!
isPing
	^ opcode = 9! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 17:09'!
isPong
	^ opcode = 10! !

!ZnWebSocketFrame methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/21/2012 16:26'!
isText
	^ opcode = 1! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
masked
	^ masked! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
masked: anObject
	masked := anObject! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
opcode
	^ opcode! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 15:08'!
opcode: anObject
	opcode := anObject! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 10:56'!
opcodeName
	| name |
	self assert: (opcode between: 0 and: 15).
	name := #(continuation text binary nil nil nil nil nil close ping pong nil nil nil nil nil) at: opcode + 1.
	^ name ifNil: [ opcode asString ] ifNotNil: [ name ]! !

!ZnWebSocketFrame methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/22/2012 10:55'!
printOn: stream
	super printOn: stream.
	stream 
		nextPut: $(; 
		print: self opcodeName;
		space;
		print: self size; nextPut: $B; 
		space;
		nextPutAll: (final ifTrue: [ 'final' ] ifFalse: [ 'non-final' ]);
		nextPut: $)! !

!ZnWebSocketFrame methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/21/2012 15:32'!
readBodyOfSize: size from: stream
	| mask |
	masked 
		ifTrue: [ 
			mask := stream next: 4.
			body := ByteArray new: size streamContents: [ :out |
				1 to: size do: [ :index |
					out nextPut: (stream next bitXor: (mask at: ((index - 1) \\ 4) + 1)) ] ] ]
		ifFalse: [
			body := stream next: size ]! !

!ZnWebSocketFrame methodsFor: 'reading' stamp: 'SvenVanCaekenberghe 9/17/2012 10:52'!
readFrom: stream
	| byte size mask |
	(byte := stream next) ifNil: [ ConnectionClosed signal ].
	final := (byte bitAt: 8) = 1.
	opcode := byte bitAnd: 127.
	(byte := stream next) ifNil: [ ConnectionClosed signal ].
	masked := (byte bitAt: 8) = 1.
	size := byte bitAnd: 127.
	size < 126
		ifFalse: [
			size = 126
				ifTrue: [ size := (stream next: 2) asInteger ]
				ifFalse: [ size := (stream next: 8) asInteger ].
			size = 0 ifTrue: [ ConnectionClosed signal ] ].
	self readBodyOfSize: size from: stream! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 20:59'!
size
	"Return the body size in bytes"
	
	^ body 
		ifNil: [ 0 ] 
		ifNotNil: [ body size ].! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/22/2012 10:28'!
text
	^ ZnUTF8Encoder new decodeBytes: body! !

!ZnWebSocketFrame methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 20:29'!
text: string
	self body: (ZnUTF8Encoder new encodeString: string)
! !

!ZnWebSocketFrame methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/21/2012 21:02'!
writeBodyOfSize: size on: stream
	| mask |
	masked
		ifTrue: [
			mask := ZnWebSocketUtils newMask.
			stream nextPutAll: mask ].
	size = 0 ifTrue: [ ^ self ].
	masked 
		ifTrue: [
			body doWithIndex: [ :each :index |
				stream nextPut: (each bitXor: (mask at: ((index - 1) \\ 4) + 1)) ] ] 
		ifFalse: [ self writeRawBodyOn: stream ]! !

!ZnWebSocketFrame methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 8/22/2012 10:58'!
writeOn: stream
	| byte size |
	byte := (final ifTrue: [ 128 ] ifFalse: [ 0 ]) + opcode.
	stream nextPut: byte.
	byte := (masked ifTrue: [ 128 ] ifFalse: [ 0 ]).
	(size := self size) < 126
		ifTrue: [ stream nextPut: (byte + size) ]
		ifFalse: [
			size < 65536
				ifTrue: [ stream nextPut: (byte + 126); nextPutAll: (size asByteArrayOfSize: 2) ]
				ifFalse: [
					self assert: size < (2 raisedTo: 31).
					stream nextPut: (byte + 127); nextPutAll: (size asByteArrayOfSize: 8) ] ].
	self writeBodyOfSize: size on: stream ! !

!ZnWebSocketFrame methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/21/2012 21:02'!
writeRawBodyOn: stream
	self isEmpty
		ifFalse: [ stream nextPutAll: body ]! !

Object subclass: #ZnWebSocketStatusHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketStatusHandler commentStamp: '<historical>' prior: 0!
I am ZnWebSocketStatusHandler, emitting some Smalltalk image status data every second as a stream of WebSocket messages.

See the class comment of ZnWebSocketDelegate for an example using ZnWebSocketStatusHandler.

Part of Zinc HTTP Components.!

!ZnWebSocketStatusHandler class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 9/7/2012 16:36'!
clientHtml
	^ '<!!DOCTYPE html>
<meta charset="utf-8" />
<title>WebSocket Smalltalk Image Status Stream (ws://localhost:1701/ws-status)</title>

<script language="javascript" type="text/javascript">

  var wsUri = "ws://localhost:1701/ws-status";
  var output, statusElement;

  function init() {
    output = document.getElementById("output");
    statusElement = document.getElementById("status");
    testWebSocket();
  }

  function testWebSocket() {
    websocket = new WebSocket(wsUri);
    websocket.onopen = function(evt) { onOpen(evt) };
    websocket.onclose = function(evt) { onClose(evt) };
    websocket.onmessage = function(evt) { onMessage(evt) };
    websocket.onerror = function(evt) { onError(evt) };
  }

  function onOpen(evt) {
    writeToScreen("CONNECTED");
    setStatus("Waiting for status data...");
   }

  function onClose(evt) {
    writeToScreen("DISCONNECTED");
  }

  function onMessage(evt) {
    setStatus(evt.data);
  }

  function onError(evt) {
    writeToScreen("<span style=''color: red''>ERROR:</span> " + evt.data);
  }

  function writeToScreen(message) {
    var pre = document.createElement("p");
    pre.style.wordWrap = "break-word";
    pre.innerHTML = message;
    output.appendChild(pre);
  }

  function setStatus(text) {
    statusElement.innerHTML = text;
  }

  window.addEventListener("load", init, false);

</script>

<h2>WebSocket Smalltalk Image Status Stream (ws://localhost:1701/ws-status)</h2>
<div><pre id="status"></pre></div>
<h3>Log</h3>
<div id="output"></div>

</html> '! !

!ZnWebSocketStatusHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2012 17:00'!
processes
	| allProcesses |
	allProcesses := (Process allSubInstances
		reject: [ :each | each isTerminated or: [ each = Processor activeProcess ] ])
			sorted: [ :a :b | a priority >= b priority ].
	^ String streamContents: [ :out |
		allProcesses doWithIndex: [ :each :index |
			out 
				print: index; nextPut: $.; space; 
				nextPutAll: each name; space;
				nextPut: $[; print: each priority; nextPut: $]; space;
				print: each; cr ] ]! !

!ZnWebSocketStatusHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2012 21:37'!
status
	^ String cr join: { 
		DateAndTime now printString. 
		self systemVersionInfo. 
		self vmStats. 
		self processes }
! !

!ZnWebSocketStatusHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2012 16:38'!
systemVersionInfo
	^ String streamContents: [ :stream |
			stream 
				print: SystemVersion current; 
				nextPutAll: ' - '; 
				nextPutAll: ZnConstants defaultServerString ]! !

!ZnWebSocketStatusHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2012 17:26'!
value: webSocket
	"I stream status messages to a WebSocket client, once every second 
	until ConnectionClosed or no longer isConnected"
	
	[ 
		self crLog: 'Started status streaming'.
		[ 
			webSocket sendMessage: self status.
			1 second asDelay wait.
			webSocket isConnected ] whileTrue ] 
		on: ConnectionClosed 
		do: [ self crLog: 'Ignoring connection close' ].
	self crLog: 'Stopping status streaming' ! !

!ZnWebSocketStatusHandler methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2012 16:23'!
vmStats
	^  [ SmalltalkImage current vm statisticsReport ]
		on: MessageNotUnderstood 
		do: [ SmalltalkImage current vmStatisticsReportString ]! !

Object subclass: #ZnWebSocketUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WebSocket-Core'!

!ZnWebSocketUtils commentStamp: '<historical>' prior: 0!
I am ZnWebSocketUtils.

Part of Zinc HTTP Components.!

!ZnWebSocketUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2012 14:15'!
containsConnectionUpgrade: headers
	^ (((headers at: 'Connection' ifAbsent: [ ^ false ]) 
			findTokens: ',') 
				collect: [ :each | each trimBoth asLowercase ]) 
					includes: 'upgrade'! !

!ZnWebSocketUtils class methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 7/19/2012 15:07'!
handshake: clientKey
	"Perform the WebSocket key handshake"
	
	"self handshake: 'dGhlIHNhbXBsZSBub25jZQ=='"
	
	| string hash |
	string := clientKey, self serverGUID.
	hash := SHA1 hashMessage: string.
	^ ZnUtils encodeBase64: hash! !

!ZnWebSocketUtils class methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 7/20/2012 15:06'!
newClientKey
	"Generate a new WebSocket client key"
	
	| rawKey |
	rawKey := self randomByteArrayOfSize: 16.
	^ ZnUtils encodeBase64: rawKey! !

!ZnWebSocketUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/20/2012 15:06'!
newMask
	^ self randomByteArrayOfSize: 4! !

!ZnWebSocketUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 20:23'!
newPingPayload
	^ self randomByteArrayOfSize: 4! !

!ZnWebSocketUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/20/2012 15:06'!
randomByteArrayOfSize: size
	^ ByteArray new: size streamContents: [ :stream |
		size timesRepeat: [ 
			stream nextPut: 256 atRandom - 1 ] ].
! !

!ZnWebSocketUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 7/19/2012 14:54'!
serverGUID
	"This is the special WebSocket server GUID"
	
	^ '258EAFA5-E914-47DA-95CA-C5AB0DC85B11'! !
